<?php
/**
 * @package WP_SoftLinks
 * @version 1.0
 * @author Frank Boon
 * @author Vincent Bosch
 */

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

class WP_SoftLinks
{

	/*
	 * Variable that witholds all the Paths & URI's that are used by this Class
	 * @access private
	 * @var array
	 */
	private $dirs;


	/*
     	 * Variable that witholds the attribute name for the html elements.
    	 * @access private
    	 * @var array
    	 */
	public $softlink_attribute;


	/*
	 * Constructor sets up {$dirs}, fires hooks and filters function
	 */
	public function __construct ()
	{
		$this->load_directories();
        	$this->load_vars();
		$this->add_actions();
		$this->add_filters();
	}


	/*
	 * Sets all the variables that are used by Softlinks
	 */
	public function load_vars()
	{
		$this->softlink_attribute = apply_filters('softlinks_attribute', 'data-wp-id');
    	}


	/*
	 * Sets all the Paths & URI's that are used by Softlinks
	 */
	public function load_directories ()
	{
		$d = &$this->dirs;

		$d = Array();

		$d['plugin_dir'] = trailingslashit( dirname(__FILE__) );

		$d['plugin_uri'] = trailingslashit( plugin_dir_url( $d['plugin_dir'] ) );
		$d['plugin_assets_uri'] = trailingslashit( $d['plugin_uri'] . 'assets' );
		$d['plugin_assets_css_uri'] = trailingslashit( $d['plugin_assets_uri'] . 'css' );
		$d['plugin_assets_js_uri'] = trailingslashit( $d['plugin_assets_uri'] . 'js' );

		$d['plugin_assets_css_softlinks_admin_uri'] = $d['plugin_assets_css_uri'] . 'softlinks_admin.css';
		$d['plugin_assets_js_wplinks_uri'] = $d['plugin_assets_js_uri'] . 'wplink.min.js';
	}


	/*
	 * Adds actions that are used by this Plugin
	 */
	public function add_actions ()
	{
		// Load Admin Styles
		add_action( 'admin_enqueue_scripts', array(&$this, 'load_admin_styles_scripts') );

        	add_action( 'admin_head', array(&$this, 'load_dynamic_scripts') );
	}


	/*
	 * Adds filters that are used by this Plugin
	 */
	public function add_filters ()
	{
		$main_post_types = array( 'media','pages','posts' );

		// Add your own Post Type to see the post ID in the columns
		$post_types = apply_filters( 'softlinks_posttypes', $main_post_types, $main_post_types );

	    	foreach($post_types as $post_type){
			// Add filter for post pages to add ID as an extra column
			add_filter('manage_' . $post_type . '_columns', array(&$this,'add_post_columns_head') );
			add_filter('manage_' . $post_type . '_custom_column', array(&$this,'add_post_columns_content'), 10, 2);
		}

		// Filter to fire media_send_to_editor() when inserting an attachment into the WP Editor
		add_filter('media_send_to_editor', array(&$this,'media_send_to_editor'), 20, 3);

		// Filter to fire replace_content() on WP Content
		add_filter( 'the_content', array(&$this,'replace_content'), 12 );

		// Filter to fire replace_content() on Widgets
		add_filter( 'widget_text', array(&$this,'replace_content'), 12 );
	}


	/*
	 * Replaces links that are sent to the WP Editor
	 * @param string $html
	 * @param integer $send_id
	 * @param array $attachment
	 * @return string
	 */
	public function media_send_to_editor($html, $send_id, $attachment) {

		$search = array(
			'src=""',
			'href=""'
		);

		$replace = array(
			$this->softlink_attribute.'="' . $send_id . '" src="',
			$this->softlink_attribute.'="' . $send_id . '" href="'
		);

		return apply_filters( 'softlinks_replace_links', str_replace($search, $replace, $html), $search, $replace, $html, $attachment );
	}


	/**
	 * Replaces the softlink attribute with the correct tags
	 * @param string $content
	 * @return string
	 */
	public function replace_content($content)
	{
		$dom = new DOMDocument;
		@$dom->loadHTML( mb_convert_encoding($content, 'HTML-ENTITIES', 'UTF-8') );

		// Fetch all the HTML tags that contain the softlink attribute
		$xpath = new DOMXPath($dom);
		$found = $xpath->query("//*[@" . $this->softlink_attribute . "]");

		// Return if not found

		if( count($found) == 0 ) return $content;

		foreach($found as $element){
			$url = NULL;

			$wp_id = $element->getAttribute( $this->softlink_attribute );
			// Remove the softlink attribute in output (cleaner)
			$element->removeAttribute( $this->softlink_attribute );

			if( is_numeric($wp_id) ){

				// Fetch Post Type from WordPress
				$post_type = get_post_type( $wp_id );

				// Check if ID is valid
				if( $post_type == "attachment" ){

					// Image? Fetch size class
					$size = NULL;

					$classes = $element->getAttribute('class');
					if( !empty($classes) ){
						$classes = explode(" ",$classes);
						foreach($classes as $class){
							if(strpos($class,'size-') !== false){
								$size = str_replace('size-','',$class);
							}
						}
					}

					if( !empty($size) ){

						// Fetch Image URL from Wordpress in the correct size
						$url = apply_filters('softlinks_image_src', wp_get_attachment_image_src($wp_id, $size), $wp_id, $size);
						$url = $url[0];
					} else {

						// Fetches Attachment URL from WordPress
						$url = apply_filters('softlinks_attachment_url', wp_get_attachment_url($wp_id), $wp_id);

					}

				} elseif($post_type) {
					// Fetches permalink from WordPress
					$url = apply_filters('softlinks_permalink', get_permalink($wp_id), $wp_id);
				}

			}

			// Set href or src attribute to the HTML tag
			$type = ($element->nodeName == 'a') ? 'href' : 'src';

			if( !is_null($url) ) $element->setAttribute($type, $url);
		}

		return $dom->saveHTML();
	}


	/**
	 * Function that loads the CSS and JavaScript that are used by this Plugin
	 */
	public function load_admin_styles_scripts ()
	{
		// Enqueue custom CSS
		wp_register_style('softlinks_admin', $this->dirs['plugin_assets_css_softlinks_admin_uri'], false, '1.0.0');
	        wp_enqueue_style('softlinks_admin');

		// Disable wplink
		wp_deregister_script( 'wplink' );

		// Register a new script file to be linked
		wp_register_script( 'wplink', $this->dirs['plugin_assets_js_wplinks_uri'], array( 'jquery', 'wpdialogs' ), false, 1 );
	}


	public function load_dynamic_scripts()
	{
		echo '<script type="text/javascript">var softlink_attribute = "'. $this->softlink_attribute .'";</script>';
	}


	/**
	 * Function that adds the WordPress ID Header to the standard WP Tables
	 * @param array $defaults
	 * @return array $new
	 */
	public function add_post_columns_head ($defaults)
	{
		$new = array();
		foreach($defaults as $key => $title) {
        	    if ($key=='title') $new['softlink_post_id'] = 'ID';
        	    $new[$key] = $title;
        	}

        	return $new;
	}


	/**
	 * Function that adds the WordPress ID content to the standard WP Tables
	 * @param string $column_name
	 * @param integer $post_ID
	 */
	public function add_post_columns_content ($column_name, $post_ID)
	{
		if ($column_name == 'softlink_post_id'){
			echo $post_ID;
		}
	}

}
